﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using Newtonsoft.Json.Linq;

namespace HeuristicLab.Services.Optimization.ControllerService.Parsers {
  public static class AlgorithmConverter {

    public static Algorithm Convert(string json) {
      var algorithm = new Algorithm();
      var o = JObject.Parse(json);
      foreach (JProperty param in o["Algorithm Parameters"]) {
        Parameter parameter = CreateParameter(param);
        algorithm.Parameters.Items.Add(parameter);
      }

      var problemParams = o["Problem Parameters"];
      // TODO: Store problem parameters
      if (problemParams != null) {
        algorithm.Problem = new Problem();
        foreach (JProperty param in problemParams) {
          Parameter parameter = CreateParameter(param);
          algorithm.Problem.Parameters.Items.Add(parameter);
        }
      }
      return algorithm;
    }

    public static Algorithm ConvertAppend(string json, Algorithm algorithm) {
      var o = JObject.Parse(json);
      foreach (JProperty param in o["Algorithm Parameters"]) {
        Parameter parameter = CreateParameter(param);
        var oldParameter = (from p in algorithm.Parameters.Items where p.Value.Name == parameter.Value.Name select p).FirstOrDefault();
        if (oldParameter != null) {
          oldParameter.Value = parameter.Value;
        }
        else {
          algorithm.Parameters.Items.Add(parameter);
        }
      }

      var problemParams = o["Problem Parameters"];
      if (problemParams != null) {
        algorithm.Problem = new Problem();
        foreach (JProperty param in problemParams) {
          Parameter parameter = CreateParameter(param);          
          var oldParameter = (from p in algorithm.Problem.Parameters.Items where p.Value.Name == parameter.Value.Name select p).FirstOrDefault();
          if (oldParameter != null) {
            oldParameter.Value = parameter.Value;
          }
          else {
            algorithm.Problem.Parameters.Items.Add(parameter);
          }
        }
      }
      return algorithm;
    }

    private static Parameter CreateParameter(JProperty property) {
      JToken token = property.Value;
      switch (token.Type) {
        case JTokenType.Integer:
        case JTokenType.Float:
          return new Parameter() { Type = ParameterType.Decimal, Value = new DecimalValue() { Name = property.Name, Value = (double)property.Value } };
        case JTokenType.Boolean:
          return new Parameter() { Type = ParameterType.Boolean, Value = new BoolValue() { Name = property.Name, Value = (bool)property.Value } };
        case JTokenType.String:
          return new Parameter() { Type = ParameterType.Type, Value = new TypeValue() { Name = property.Name, Value = (string)property.Value } };                  
        default:
          throw new Exception("Unhandled datatype: " + property.Type);
      }
    }
  }
}
